from argparse import ArgumentParser
from functools import partial
from importlib import metadata

from rich.console import Group
from rich.table import Column, Table

from ..arguments import debug_arg, export_arg, json_arg, profile_arg, team_arg, config_arg
from .base import BaseCommand
from .config import ConfigCommand
from .dashboard import DashboardCommand
from .help import HelpCommand
from .install import InstallCommand
from .local import LocalCommand
from .monitor import MonitorCommand
from .monitors import MonitorsCommand
from .playbook import PlaybookCommand
from .playbooks import PlaybooksCommand
from .repo import RepoCommand
from .report import ReportCommand
from .reports import ReportsCommand
from .repos import ReposCommand
from .run import RunCommand
from .scan import ScanCommand
from .scans import ScansCommand
from .team import TeamCommand
from .teams import TeamsCommand
from .update import UpdateCommand
from .shards import ShardsCommand
from .settings import SettingsCommand
from .template import TemplateCommand
from .templates import TemplatesCommand
from .width import WidthCommand
from .whoami import WhoamiCommand

VERSION = metadata.version("satori-ci")


class RootCommand(BaseCommand):
    subcommands = (
        ConfigCommand,
        RunCommand,
        ReportCommand,
        ReportsCommand,
        MonitorCommand,
        MonitorsCommand,
        RepoCommand,
        ReposCommand,
        ScanCommand,
        PlaybookCommand,
        PlaybooksCommand,
        DashboardCommand,
        TeamCommand,
        TeamsCommand,
        HelpCommand,
        UpdateCommand,
        LocalCommand,
        ScansCommand,
        ShardsCommand,
        InstallCommand,
        WidthCommand,
        WhoamiCommand,
        SettingsCommand,
        TemplateCommand,
        TemplatesCommand,
    )
    name = "satori"
    global_options = (profile_arg, json_arg, debug_arg, export_arg, team_arg, config_arg)
    default_subcommand = DashboardCommand

    def register_args(self, parser: ArgumentParser):
        parser.add_argument(
            "-v", "--version", action="version", version=f"%(prog)s {VERSION}"
        )
        parser.add_argument(
            "--pending", action="store_true", help="Show pending actions"
        )
        parser.add_argument("--public", action="store_true", help="Fetch public")

    def help(self):
        def add_rows(table: Table, rows):
            for row in rows:
                table.add_row(*row)

        def cols():
            return Column(ratio=1), Column(ratio=1)

        HelpTable = partial(Table, expand=True, show_header=False)
        tables = []

        tables.append(
            intro := HelpTable(*cols(), title="Install, Update and Configure")
        )
        rows = [
            ("pip3 install satori-ci", "Install the latest version"),
            ("satori update", "Update to the latest version"),
            (
                'satori config token "USERTOKEN"',
                "Configure your user token as your default profile",
            ),
            (
                'satori config token "TEAMTOKEN" --profile TEAM',
                "Configure your team token on your team profile",
            ),
        ]
        add_rows(intro, rows)

        tables.append(run_async := HelpTable(*cols(), title="Run asyncronously"))
        rows = [
            (
                "satori run ./",
                "Upload the current dir and run the playbook .satori.yml",
            ),
            ("satori run playbook.yml", "Upload the playbook and run it"),
            (
                'satori run ./ --playbook="satori://..."',
                "Upload the current dir and run the specified playbook",
            ),
        ]
        add_rows(run_async, rows)

        tables.append(
            run_sync := HelpTable(
                *cols(), title="Run syncronously using these parameters"
            )
        )
        rows = [
            ("--sync", "Show the result"),
            ("--report", "Show the report"),
            ("--output", "Show the output"),
            (
                "--files",
                "Download the files created if the setting files was set to True",
            ),
        ]
        add_rows(run_sync, rows)

        tables.append(
            params := HelpTable(*cols(), title="Run playbooks with variables")
        )
        rows = [
            (
                '--data VAR="This is the value of VAR"',
                "Provide values for the undefined playbook variables",
            )
        ]
        add_rows(params, rows)

        tables.append(playbooks := HelpTable(*cols(), title="Playbooks"))
        rows = [
            ("satori playbooks", "List your private playbooks"),
            ("satori playbooks --public", "List the public playbooks"),
            ("satori playbook ID", "Show a certain playbook"),
            ("satori playbook ID public", "Toggles the playbook's visibility"),
            ("satori playbook ID delete", "Delete the playbook"),
        ]
        add_rows(playbooks, rows)

        tables.append(dashboards := HelpTable(*cols(), title="Dashboards"))
        rows = [
            ("satori", "Show your general dashboard"),
            ("satori team TEAM", "Show your TEAM dashboard"),
        ]
        add_rows(dashboards, rows)

        tables.append(reports := HelpTable(*cols(), title="Reports"))
        rows = [
            ("satori reports", "List reports"),
            ("satori report ID", "Show the report ID"),
            ("satori report ID --json", "Show the JSON of the report ID"),
            ("satori report ID output", "Show the output of the report ID"),
            (
                "satori report ID output --json",
                "Show the JSON's output of the report ID",
            ),
            (
                "satori report ID files",
                "Download the files created (if Files was set to True in settings)",
            ),
            ("satori report ID public", "Toggles the report's visibility"),
            ("satori report ID stop", "Stop the current report execution"),
            ("satori report ID delete", "Delete the report ID"),
        ]
        add_rows(reports, rows)

        tables.append(repos := HelpTable(*cols(), title="Repos"))
        rows = [
            ("satori repos", "List the repositories connected to CI or tested"),
            (
                "satori repo GithubUser/Repo",
                "Shows the repository Visibility, CI, Playbook, Status, Result and its team.",
            ),
            (
                "satori repo GithubUser/Repo run",
                "Run the repository's playbook on the latest commit",
            ),
            (
                'satori repo GithubUser/Repo run --playbook="satori://..."',
                "Run another playbook on the latest commit",
            ),
            (
                "satori repo GithubUser/Repo commits",
                "Show the list of commits and the reports associated",
            ),
        ]
        add_rows(repos, rows)

        tables.append(monitors := HelpTable(*cols(), title="Monitors"))
        rows = [
            ("satori monitors", "List monitors"),
            ("satori monitor ID", "List the reports associated to a monitor ID"),
            ("satori monitor ID stop", "Stop a monitor ID"),
            ("satori monitor ID start", "Start a monitor ID"),
            (
                "satori monitor ID clean",
                "Delete the reports associated to the monitor ID",
            ),
            ("satori monitor ID delete", "Delete the monitor ID"),
            ("satori monitor ID public", "Toggles the monitor's visibility"),
        ]
        add_rows(monitors, rows)

        tables.append(scans := HelpTable(*cols(), title="Scans"))
        rows = [
            ("satori scans", "List scans"),
            (
                "satori scan GithubUser/Repo [-c N]",
                "Scan the Github repository with the repository's playbook a coverage of 1 (default) to 100",
            ),
            (
                'satori scan GithubUser/Repo [--playbook="satori://..."]',
                "Scan the Github repository with a different playbook",
            ),
            (
                "satori scan GithubUser/Repo check-commits",
                "Get the repository commits before scanning",
            ),
            (
                "satori scan GithubUser/Repo check-forks",
                "Get the repository forks before scanning",
            ),
            (
                "satori scan ID status",
                "Show the status of the scan",
            ),
            ("satori scan ID stop", "Stop the scan on the repo"),
            (
                "satori scan ID start",
                "Start a previously stopped scan",
            ),
            (
                "satori scan ID clean",
                "Delete the reports associated to the scan",
            ),
            ("satori scan ID delete", "Delete the scan"),
            ("satori scan ID public", "Toggle the scan's visibility"),
        ]
        add_rows(scans, rows)

        tables.append(teams := HelpTable(*cols(), title="Teams"))
        rows = [
            ("satori teams", "List your teams"),
            ("satori team TEAM", "Show the TEAM dashboard"),
            ("satori team TEAM create", "Create a new team named TEAM"),
            ("satori team TEAM members", "List your TEAM members"),
            ("satori team TEAM monitors", "List your TEAM monitors"),
            ("satori team TEAM repos", "List your TEAM repositories"),
            ("satori team TEAM reports", "List your TEAM reports"),
            ("satori team TEAM settings", "List your TEAM settings"),
            ("satori team TEAM get_config NAME", "Show your TEAM's config setting"),
            ("satori team TEAM set_config NAME VALUE", "Set your TEAM CONFIG setting"),
            (
                'satori team TEAM add --github="GithubUser"',
                "Owners and admins can add users via Github to the TEAM",
            ),
            ('--role="READ"', "use the role READ (default) or ADMIN"),
            (
                'satori team TEAM add --email="usr@example.com"',
                "Owners and admins can add users via Email to the TEAM",
            ),
            ('--role="READ"', "use the role READ (default) or ADMIN"),
            (
                'satori team TEAM add --monitor="MONITORID"',
                "Add the monitor ID to your TEAM",
            ),
            (
                'satori team TEAM add --repo="GithubUser/repo"',
                "Add the repo to your TEAM",
            ),
            (
                'satori team TEAM del --github="GithubUser"',
                "Delete the GithubUser from your TEAM",
            ),
            (
                'satori team TEAM del --email="usr@example.com"',
                "Delete the email from the TEAM",
            ),
            (
                'satori team TEAM del --repo="GithubUser/repo"',
                "Delete the repo from the TEAM",
            ),
            (
                'satori team TEAM del --monitor="MONITORID"',
                "Delete the monitor from the TEAM",
            ),
            ("satori team TEAM delete", "Delete the TEAM"),
        ]
        add_rows(teams, rows)

        tables.append(shards := HelpTable(*cols(), title="Shards"))
        rows = [
            ("satori shards --shard X/Y --input INPUT", "Divide massive datasets into smaller chunks for distributed processing"),
            ("--shard X/Y", "Shard index X out of Y total shards (required)"),
            ("--input INPUT", "Input file path or direct IP/CIDR/range/domain/URL (required)"),
            ("--exclude PATH or ENTRY", "Exclusion file path or direct IP/CIDR/range/domain/URL to exclude"),
            ("--seed N", "Seed for deterministic pseudorandom distribution (default: 1)"),
            ("--results PATH", "Output file path (writes to stdout if omitted)"),
        ]
        add_rows(shards, rows)

        return Group(*tables)



from argparse import ArgumentParser
from typing import Literal, TypeAlias, get_args

from rich.prompt import Prompt

from satoricli.api import client
from satoricli.cli.utils import console

from .base import BaseCommand

OPTIONS: TypeAlias = Literal[
    "default",
    "datadog",
    "discord",
    "email",
    "slack",
    "telegram",
]
KEYS: TypeAlias = Literal[
    "default",
    "datadog_api_key",
    "datadog_site",
    "discord",
    "email",
    "slack_workspace",
    "slack_channel",
    "telegram",
]


class SettingsCommand(BaseCommand):
    name: str = "settings"

    def register_args(self, parser: ArgumentParser) -> None:
        _ = parser.add_argument(
            "key",
            metavar="KEY",
            choices=get_args(OPTIONS),
            nargs="?",
        )

    def __call__(self, key: OPTIONS | None = None, **kwargs) -> None:
        self.run_settings(key, kwargs.get("team") or "Private")

    def run_settings(
        self,
        key: OPTIONS | None = None,
        team: str = "Private",
    ) -> None:
        self.team: str = team
        if key == "default":
            logs: dict[str, bool] = client.get(f"/teams/{self.team}/logs").json()
            filtered = list(filter(lambda x: logs[x] is True, logs))
            console.print("Default notification method: " + " | ".join(filtered))
            self.ask_default()
            return
        if key is not None:
            val = self.get_setting(key)
            console.print(f"[green]Current value: {val}")
            if key == "datadog":
                self.ask_datadog()
            elif key == "discord":
                self.ask_discord()
            elif key == "email":
                self.ask_email()
            elif key == "slack":
                self.ask_slack()
            elif key == "telegram":
                self.ask_telegram()
            return
        while True:
            display_options: tuple[OPTIONS] = get_args(OPTIONS)
            for i, option in enumerate(display_options, 0):
                val = self.get_setting(option)
                console.print(f"{i}. {option}: {val}")

            console.print()
            console.print("9. exit")
            console.print()

            choices = [str(i) for i in range(len(display_options))] + ["9"]
            choice = Prompt.ask("Choose an option", choices=choices)
            match choice:
                case "9":
                    break
                case "0":
                    self.ask_default()
                case "1":
                    self.ask_datadog()
                case "2":
                    self.ask_discord()
                case "3":
                    self.ask_email()
                case "4":
                    self.ask_slack()
                case "5":
                    self.ask_telegram()
                case _:
                    pass

    def get_by_key(self, key: str):
        val = client.get(f"/teams/{self.team}/config/{key}").text
        return val

    def get_setting(self, key: OPTIONS):
        get_config = []
        if key == "default":
            logs: dict[str, bool] = client.get(f"/teams/{self.team}/logs").json()
            filtered = list(filter(lambda x: logs[x] is True, logs))
            return " | ".join(filtered) if filtered else "not set"
        if key == "datadog":
            get_config = ["datadog_api_key", "datadog_site"]
        if key == "discord":
            get_config = ["discord_channel"]
        if key == "email":
            get_config = ["notification_email"]
        if key == "slack":
            get_config = ["slack_workspace", "slack_channel"]
        if key == "telegram":
            get_config = ["telegram_channel"]
        values = []
        for config in get_config:
            val = client.get(f"/teams/{self.team}/config/{config}").text
            values.append(val or "not set")
        return " | ".join(values)

    def set_setting(self, key: str, value: str) -> None:
        _ = client.put(f"/teams/{self.team}/config", json={"name": key, "value": value})

    def ask_setting(
        self, key: str, prompt: str, choices: list[str] | None = None
    ) -> None:
        self.set_setting(key, Prompt.ask(prompt, choices=choices))

    def ask_default(self, value: str | None = None) -> None:
        default_dict: dict[str, bool] = {
            "slack": False,
            "discord": False,
            "datadog": False,
            "email": False,
            "telegram": False,
        }
        user_input: str = value or Prompt.ask(
            "Please select a default notification method",
            choices=[k for k in default_dict.keys()] + ["none"],
        )
        if user_input != "none":
            default_dict[user_input] = True
        _ = client.put(f"/teams/{self.team}/logs", json=default_dict)
        console.print("Default notification method updated\n")

    def ask_datadog(self) -> None:
        console.print(
            "To set up Datadog, you need to get an API key from your Datadog account.\n\n"
            + "You can find your API key in the Datadog API keys page(create a new API key if needed): \n"
            + "[cyan b]https://app.datadoghq.com/organization-settings/api-keys[/]"
        )
        self.ask_setting("datadog_api_key", "Enter your Datadog API key")
        console.print("Datadog API key updated")
        self.ask_setting(
            "datadog_site",
            "Enter your Datadog region",
            ["us1", "us3", "us5", "eu1", "ap1", "ap2", "us1-fed"],
        )
        console.print("Datadog site updated\n")

    def ask_discord(self) -> None:
        console.print(
            "To set up Discord, you need to add the bot to your server and get the channel ID.\n\n"
            + "Invite the bot to the server with the following link:\n"
            + "[cyan b]https://discord.com/api/oauth2/authorize?client_id=1038196776752128131&permissions=2048&scope=bot[/]"
        )
        _ = input(
            "After inviting the bot to your channel, press any key to continue..."
        )
        console.print(
            "\nTo enable Developer mode and get your channel ID follow these instructions:\n"
            + "[cyan b]https://support.discord.com/hc/en-us/articles/206346498-Where-can-I-find-my-User-Server-Message-ID[/]"
        )
        self.ask_setting(
            "discord_channel", "Copy and paste the Discord channel ID here"
        )
        console.print("Discord channel updated\n")

    def ask_email(self) -> None:
        self.ask_setting("notification_email", "Notification emails (comma separated)")
        console.print("Notification email updated")

    def ask_slack(self) -> None:
        console.print(
            "To set up Slack, you need to add the bot to your workspace and invite it to the channel.\n\n"
            + "You can invite the bot to the workspace by opening this link:\n"
            + "[cyan b]https://satori.ci/api/authorise/slack[/]\n\n"
            + "You can invite the bot to the channel by typing the following command: [b]/invite @SatoriCIBot[/]"
        )
        _ = input(
            "After inviting the bot to your channel, press any key to continue..."
        )
        console.print(
            "\nYou can find the workspace ID in the URL of your workspace, e.g.\n"
            + "[cyan b]https://app.slack.com/client/<workspace_id>/channels[/]"
        )
        self.ask_setting("slack_workspace", "Enter your Slack workspace ID")
        console.print("Slack workspace updated")
        self.ask_setting("slack_channel", "Enter your Slack channel ID(e.g. #general)")
        console.print("Slack channel updated\n")

    def ask_telegram(self) -> None:
        console.print(
            "To set up Telegram, you need to add the bot to your channel.\n\n"
            + "Invite @satori_ci_bot as a new member to your channel."
        )
        _ = input(
            "After inviting the bot to your channel, press any key to continue..."
        )
        console.print(
            "Obtain the Channel ID: access your channel via the web at Telegram Web."
            + "The Channel ID is the number that appears after the # in the URL"
        )
        self.ask_setting("telegram_channel", "Insert the Channel ID(e.g. -15050500050)")
        console.print("Telegram channel updated\n")
